#!/usr/bin/perl
use lib '/home/topeg/PERL/AA_LIBS';
use strict;
use warnings;
use Data::Dumper;
use Linux::UVC;
use Gtk2 -init;
use FindBin '$Bin';

my @devs=Linux::UVC->devices();
my $dev=undef;

my $gui=GUI->new("$Bin/controll.glade");
$gui->signal_connect(do_exit => sub{ Gtk2->main_quit; });
$gui->signal_connect(do_select => sub
  {
    my $gui=shift;
    my $pos=shift;
    $dev=$devs[$pos];
    #print Dumper($dev);
    $gui->close_selectwin();
    $gui->set_device_params($dev->name,$dev->device,$dev->id);

    for my $ctrl ($dev->controlls())
    { $gui->add_controll($ctrl->name,$ctrl->type,$ctrl->min,$ctrl->max,$ctrl->step,$ctrl->value); }

    $gui->show_mainwin();
  });

$gui->signal_connect(do_changed => sub
  {
    my ($gui,$name,$val)=@_;
    return unless($dev);
    my $ctrl=$dev->controll_by_name($name);
    return unless($ctrl);

    $ctrl->set($val);
    #print "$name => $val\n";
  });

$gui->signal_connect(do_reset => sub
  {
    $_[0]->set_control_val($_->name,$_->default) for($dev->controlls())
  });

$gui->add_device($_->name()) for(@devs);

$gui->show_selectwin();

Gtk2->main;

########################################################################
########################################################################
########################################################################

package GUI;
use strict;
use warnings;
use Gtk2;

use Glib::Object::Subclass
  Gtk2::Builder::,
  signals => {
    do_changed => {
      flags => ['run-last'],
      param_types => [qw/Glib::String Glib::String/],
    },
    do_exit => {
      flags => ['run-last'],
      param_types => [ ],
    },
    do_reset => {
      flags => ['run-last'],
      param_types => [ ],
    },
    do_select => {
      flags => ['run-last'],
      param_types => [qw/Glib::String/],
    },
  },
  properties => [];

sub new
{
  my $class=shift;
  my $file=shift;
  my $self=$class->SUPER::new();

  if(defined($file))
  {
    my $ff=eval{local ($/,@ARGV)=(undef,$file); <>; };
    return undef unless($self->add_from_string($ff));
  }
  #else
  #{ return undef unless($self->add_from_string(eval{local $/=undef; <DATA> })); }


  $self->{params}=[];

  my $sel = $self->dev_sel();
  my $store = Gtk2::ListStore->new(qw/Glib::String/);
  $sel->set_model($store);
  my $renderer = Gtk2::CellRendererText->new();
  $sel->pack_start($renderer, 1);
  $sel->add_attribute($renderer,'text',0);

  $self->connect_signals($self);
  return $self;
}

sub mainwin   { return $_[0]->get_object('mainwin'); }
sub selwin    { return $_[0]->get_object('selectwin'); }
sub status_bar{ return $_[0]->get_object('status'); }
sub param_box { return $_[0]->get_object('params'); }
sub dev_sel   { return $_[0]->get_object('device_select'); }
sub dev_name  { return $_[0]->get_object('device_name'); }
sub dev_device{ return $_[0]->get_object('device_device'); }
sub dev_id    { return $_[0]->get_object('device_id'); }

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
#||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||#
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#

sub add_device
{
  my $self=shift;
  my $name=shift;
  my $sel=$self->dev_sel();
  my $model=$sel->get_model();
  $sel->append_text($name);
  $sel->set_active(0);
}

sub add_controll
{
  my $self=shift;
  my ($name,$type,$min,$max,$step,$now)=@_;

  my $box=$self->param_box();

  my $hbox=Gtk2::HBox->new();

  my $label=Gtk2::Label->new($name);
  $label->set_size_request(100, -1);
  $label->set_alignment(1, 1);
  $hbox->pack_start($label, 0, 1, 0);


  my $pos=@{$self->{params}};

  my $elm=undef;

  if($type eq 'Boolean')
  {
    $elm=Gtk2::CheckButton->new ('ON/OFF');
    $elm->set_active($now?1:0);
    $hbox->pack_start($elm, 1, 1, 0);
    $elm->signal_connect(toggled => sub{$self->_param_changed($pos)} );
  }
  elsif($type eq 'Dword')
  {
    $elm=Gtk2::HScale->new_with_range($min, $max, $step);
    $elm->set_value($now);
    $hbox->pack_start($elm, 1, 1, 0);
    $elm->signal_connect('value-changed' => sub{$self->_param_changed($pos)} );
  }

  push(@{$self->{params}},[$name,$type,$elm]);

  $box->pack_start($hbox,0,1,0);
}

sub _param_changed
{
  my $self=shift;
  my $pos=shift;
  my ($name,$type,$elm)=@{$self->{params}->[$pos]};

  my $val=undef;

  if($type eq 'Boolean')
  { $val=$elm->get_active(); }
  elsif($type eq 'Dword')
  { $val=$elm->get_value(); }
  $self->signal_emit('do_changed',$name,$val);
}

sub set_control_val
{
  my $self=shift;
  my $name=shift;
  my $val=shift;

  for my $v (@{$self->{params}})
  {
    if($v->[0] eq $name)
    {
      if($v->[1] eq 'Boolean')
      { $v->[2]->set_active($val); }
      elsif($v->[1] eq 'Dword')
      { $v->[2]->set_value($val); }
      last;
    }
  }
}

sub set_device_params
{
  my $self=shift;
  my ($name,$device,$id)=@_;
  $self->dev_name()->set_label($name);
  $self->dev_device()->set_label($device);
  $self->dev_id()->set_label($id);
}
#-----------------------------------------------------------------------

sub on_exit
{
  my $self=pop;
  $self->mainwin()->destroy();
  $self->signal_emit('do_exit');
  return 1;
}

sub on_reset
{
  my $self=pop;
  $self->signal_emit('do_reset');
}

sub on_select
{
  my $self=pop;
  my $sel=$self->dev_sel()->get_active() // 0;
  $self->signal_emit('do_select',$sel);
}

#.......................................................................
#-----------------------------------------------------------------------
#'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

sub show_mainwin  { $_[0]->mainwin()->show_all(); }
sub close_mainwin { $_[0]->mainwin()->destroy(); }

sub show_selectwin  { $_[0]->selwin()->show_all(); }
sub close_selectwin { $_[0]->selwin()->destroy(); }

1;